"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ReportExecutionURLGenerateTypedDict(TypedDict):
    expires_in: NotRequired[int]
    r"""The URL expiration time, in minutes."""


class ReportExecutionURLGenerate(BaseModel):
    expires_in: Optional[int] = 5
    r"""The URL expiration time, in minutes."""
