"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .taxidkind import TaxIDKind
from gr4vy.types import BaseModel
from gr4vy.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing_extensions import Annotated, TypedDict


class TaxIDTypedDict(TypedDict):
    value: str
    r"""The tax ID for the buyer."""
    kind: TaxIDKind


class TaxID(BaseModel):
    value: str
    r"""The tax ID for the buyer."""

    kind: Annotated[TaxIDKind, PlainValidator(validate_open_enum(False))]
