"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class GooglePaySessionRequestTypedDict(TypedDict):
    origin_domain: str
    r"""The domain on which Google Pay is being loaded."""


class GooglePaySessionRequest(BaseModel):
    origin_domain: str
    r"""The domain on which Google Pay is being loaded."""
