"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .giftcardsummary import GiftCardSummary, GiftCardSummaryTypedDict
from gr4vy.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class GiftCardSummariesTypedDict(TypedDict):
    items: List[GiftCardSummaryTypedDict]
    r"""A list of items returned for this request."""


class GiftCardSummaries(BaseModel):
    items: List[GiftCardSummary]
    r"""A list of items returned for this request."""
