"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class ReportExecutionContextTypedDict(TypedDict):
    reference_timestamp: datetime
    r"""The reference timestamp for the report execution context."""
    reference_timezone: str
    r"""The reference timezone for the report execution context."""


class ReportExecutionContext(BaseModel):
    reference_timestamp: datetime
    r"""The reference timestamp for the report execution context."""

    reference_timezone: str
    r"""The reference timezone for the report execution context."""
